#Symmetry detection in irregular static patterns - Dec 2015
#Static pattern with different contrast levels - Sept 2017
#Each contrast level is converted into noise in a pseudorandom order
#So that there are as equal as possible numbers of each contrast level in symmetry and noise.

from psychopy import prefs
prefs.general['audioLib'] = ['pygame']
from psychopy import visual, event, core, data, gui, misc, sound, logging
from psychopy.hardware import crs
import symFunctions16, time, copy, random, os, cPickle
import numpy as np
import dataTest

DEBUG = True

#Create a dialog box for ppt information
try:
    info=misc.fromFile('contrStaticLL.pickle')
except:
    info = {'participant' : 'RJS', 'displayT':34, 'density':40, 'contrast levels' : 4}
info['dateStr']=time.strftime("%b%d_%H%M", time.localtime())
dlg = gui.DlgFromDict(info, title='Experiment', fixed=['dateStr'])
if dlg.OK:
    misc.toFile('contrStaticLL.pickle', info)
else:
    core.quit()

#Basic info
info['size'] = 320#350 #one side of the square
info['dotSize'] = 6 #diameter in pixels
#check number of dots is even
if info['density'] % 2 != 0:
    print 'ERROR: Must have an even number of dots'
    core.quit()
info['halfnDots'] = info['density']/2 #20 #half the total number of nots, to prevent odd number problems
info['bubbleSize'] = 12#14 #the size of the bubble each dot can move around in
info['jitter'] = 0
info['minDist'] = (info['bubbleSize']*2) + (info['dotSize']/2)

#contrasts
if info['contrast levels'] == '10.2':
    info['contrast'] = [0.2]
if info['contrast levels'] == '10.9':
    info['contrast'] = [0.9]
if info['contrast levels'] == '11.0':
    info['contrast'] = [1.0]
if info['contrast levels'] == '2':
    info['contrast'] = [0.2,0.9]
if info['contrast levels'] == '4':
    info['contrast'] = [0.2,0.433,0.666,0.9]
if info['contrast levels'] == '5':
    info['contrast'] = [0.2,0.375,0.55,0.725,0.9]
if info['contrast levels'] == '10':
    info['contrast'] = [0.2,0.2777,0.3554,0.4331,0.5108,0.5885,0.6662,0.7439,0.8216,0.9]
multiplier = info['density']/len(info['contrast'])

info['contrasts'] = []
for n in range(multiplier):
    random.shuffle(info['contrast'])
    temp = copy.copy(info['contrast'])
    info['contrasts'] += temp

#Staircase information
info['nTrials'] = 75
info['nReversals'] = 8
info['stepSizes'] = [8,8,4,4,2,2,1,1] #jitter in pixels
info['minVal'] = 0
info['maxVal'] = info['halfnDots']
info['startVal'] = info['halfnDots']
info['nUp'] = 1
info['nDown'] = 3
info['feedback'] = True

#Create a window
if DEBUG == True:
    myWin = visual.Window(size = (800,800), monitor = 'testMonitor', units = 'pix', fullscr = False, allowGUI = False)
    #info['displayT'] = 30 #500ms at 60Hz
    info['ISI'] = 30
if DEBUG == False:
    myWin = visual.Window(size = [1024,768], screen = 1, useFBO = True, fullscr = True, allowGUI = False, monitor = 'Topaz', units = 'pix')
    #info['displayT'] = 34 #400ms at 85Hz
    #info['displayT'] = 68 #800ms at 85Hz
    #bits = crs.BitsSharp(myWin, mode = 'bits++')
    info['ISI'] = 34


#sounds
beep = sound.Sound('800', secs=0.01)
boop = sound.Sound('300', secs=0.1)

#Checking Responses
def checkCorrect (keys):
    for key in keys:
        if key in ['q', 'escape']:
            core.quit()
        elif key in ['num_1', 'num_3']:
            if (key in ['num_1']) and order==1:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1 #subject perceives the first image as more symmetrical
            if (key in ['num_3']) and order==1:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0 #subject perceives the second image as more symmetrical
            if (key in ['num_1']) and order==2:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0
            if (key in ['num_3']) and order==2:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1
            else:
                print "hit 1 or 3 (or q) (you hit %s)" %key
                return None

#Create multiple staircases in order to interleave them
stairs = []
conditions = ['shiftHigh', 'shiftLow'] #one starting with no jitter and one starting with max jitter

for thisCond in conditions:
    thisInfo = copy.copy(info)
    if thisCond == 'shiftHigh':
        thisInfo['cond'] = 'shiftHigh'
        thisInfo['nShift'] = info['maxVal']
        thisInfo['startVal'] = info['maxVal']
    if thisCond == 'shiftLow':
        thisInfo['cond'] = 'shiftLow'
        thisInfo['startVal'] = info['minVal']
        thisInfo['nShift'] = info['minVal']
    
    #staircase info
    #info for this staircase
    thisStair = dataTest.StairHandler(startVal = thisInfo['startVal'],
                                                nReversals = info['nReversals'],
                                                stepSizes = info['stepSizes'],
                                                stepType = 'lin',
                                                nTrials = info['nTrials'],
                                                nUp = info['nUp'],
                                                nDown = info['nDown'],
                                                extraInfo = thisInfo,
                                                minVal = info['minVal'],
                                                maxVal = info['maxVal'],
                                                        )
    stairs.append(thisStair)

#fixation
fixation = visual.Circle(myWin, radius = 3.5, lineColor = 'black', fillColor = 'black', contrast = 0.3, interpolate = True)
fixation.setUseShaders(True)
#instructions
startMessage = visual.TextStim(myWin, pos = (0.0, -150), height = 25, rgb = 1,
                text = 'Please press 1 or 3 to indicate whether the first or second image appears more symmetrical. Press any key to continue')
startMessage.draw()
fixation.draw()
myWin.flip()
event.waitKeys()
myWin.flip()
#create dot patterns
circles = {}
foilcircles = {}
#test coordinates
rightxy, leftxy, xy = symFunctions16.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'])
xy[:,0] -= info['size']/2
xy[:,1] -= info['size']/2
#foil coordinates
foilrightxy, foilleftxy, foilxy = symFunctions16.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'])
foilxy[:,0] -= info['size']/2
foilxy[:,1] -= info['size']/2

#Go through the trials
for trialN in range(info['nTrials']):
    random.shuffle(stairs)
    for thisStair in stairs:
        thisShift = info['maxVal'] - thisStair.next()
        print 'shift', thisShift
        
        #create new patterns
        rightxy, leftxy, xy = symFunctions16.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], jitter = 0)
        foilrightxy, foilleftxy, foilxy = symFunctions16.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], jitter = 0)
        xy, noiseIndex = symFunctions16.sequentialNoise(thisShift, xy, info['bubbleSize'], info['dotSize'], info['size'])
        foilxy, foilNoiseIndex = symFunctions16.sequentialNoise(info['maxVal'], foilxy, info['bubbleSize'], info['dotSize'], info['size'])
        xy[:,0] -= info['size']/2
        xy[:,1] -= info['size']/2
        foilxy[:,0] -= info['size']/2
        foilxy[:,1] -= info['size']/2
        for n in range(info['density']):
            circles[n] = visual.GratingStim(myWin, tex = 'none', mask = 'circle', size = info['dotSize'], units = 'pix', pos = xy[n])
            #circles[n] = visual.Circle(myWin, radius = info['dotSize']/2, units = 'pix', fillColor = 'white', pos = xy[n], interpolate = True)
            circles[n].setUseShaders(True)
            foilcircles[n] = visual.GratingStim(myWin, tex = 'none', mask = 'circle', size = info['dotSize'], units = 'pix', pos = foilxy[n])
            #foilcircles[n] = visual.Circle(myWin, radius = info['dotSize']/2, units = 'pix', fillColor = 'white', pos = foilxy[n], interpolate = True)
            foilcircles[n].setUseShaders(True)
        
        signalIndex = [x for x in range(info['halfnDots']*2) if x not in noiseIndex]
        #foilSignalIndex = [x for x in range(halfnDots*2) if x not in foilNoiseIndex]

        order = random.randint(1,2)
        #present stimuli
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
                #bits.beep(freq = 1200, dur = 0.3)
        
        #set contrast
        for n in range(info['halfnDots']):
            circles[n].contrast = info['contrasts'][n]
            circles[n+info['halfnDots']].contrast = info['contrasts'][n]
            foilcircles[n].contrast = info['contrasts'][n]
            foilcircles[n+info['halfnDots']].contrast = info['contrasts'][n]

        for frameN in range(info['displayT']):
            for n in range(info['halfnDots']):
                if order == 1:
                    circles[n].draw()
                    circles[n+info['halfnDots']].draw()
                if order == 2:
                    foilcircles[n].draw()
                    foilcircles[n+info['halfnDots']].draw()
            
            fixation.draw()
            myWin.flip()
        myWin.getMovieFrame()
        myWin.saveMovieFrames('contr1.jpg')

        #event.waitKeys()
        #ISI
        for frameN in range(info['ISI']):
            fixation.draw()
            myWin.flip()
        #second interval
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
            #bits.beep(freq = 1200, dur = 0.3)
        for frameN in range(info['displayT']):
            for n in range(info['halfnDots']):
                if order == 2:
                    circles[n].draw()
                    circles[n+info['halfnDots']].draw()
                if order == 1:
                    foilcircles[n].draw()
                    foilcircles[n+info['halfnDots']].draw()
            
            fixation.draw()
            myWin.flip()
        myWin.getMovieFrame()
        myWin.saveMovieFrames('contr2.jpg')
        #event.waitKeys()
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

        #clear the screen
        fixation.draw()
        myWin.flip()
        
        #collect response
        thisResp = None
        while thisResp==None:
            keys = event.waitKeys()
            thisResp = checkCorrect(keys)
        #print 'thisResp', thisResp
        
        thisStair.addData(thisResp)
                
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

#Saving data
if not os.path.isdir('irrStatic_%s' %info['participant']):
    os.mkdir('irrStatic_%s' %info['participant'])

for thisStair in stairs:
    fName = 'irrStatic_%s//irrStaticContrModStatic_levels%s_%dms_%ddots_%s_%s_%s' %(thisStair.extraInfo['participant'], thisStair.extraInfo['contrast levels'], thisStair.extraInfo['displayT'],
                    thisStair.extraInfo['density'],thisStair.extraInfo['cond'], thisStair.extraInfo['participant'], info['dateStr'])
    
    thisStair.saveAsExcel(fileName = fName, sheetName = 'RawData', matrixOnly = False, appendFile = True)
    thisStair.saveAsPickle(fName)
core.quit()







