#Symmetry detection in irregular static patterns - Dec 2015

from psychopy import prefs
prefs.general['audioLib'] = ['pygame']
from psychopy import visual, event, core, data, gui, misc, sound, logging
from psychopy.hardware import crs
import symFunctions10, time, copy, random, os, cPickle
import numpy as np
import dataTest

DEBUG = True

#Create a dialog box for ppt information
try:
    info=misc.fromFile('symStaticLL.pickle')
except:
    info = {'participant' : 'RJS', 'lifeTime' : 27,  'displayT':34, 'density':40}
info['dateStr']=time.strftime("%b%d_%H%M", time.localtime())
dlg = gui.DlgFromDict(info, title='Experiment', fixed=['dateStr'])
if dlg.OK:
    misc.toFile('symStaticLL.pickle', info)
else:
    core.quit()

#Basic info
info['size'] = 320#350 #one side of the square
info['dotSize'] = 6 #diameter in pixels
#check number of dots is even
if info['density'] % 2 != 0:
    print 'ERROR: Must have an even number of dots'
    core.quit()
info['halfnDots'] = info['density']/2 #20 #half the total number of nots, torevent odd number problems
info['bubbleSize'] = 12#14 #the size of the bubble each dot can move around in
info['jitter'] = 0
#info['lifeTime'] = info['bubbleSize']*2
info['minDist'] = (info['bubbleSize']*2) + (info['dotSize']/2)

#Staircase information
info['nTrials'] = 75
info['nReversals'] = 8
info['stepSizes'] = [8,8,4,4,2,2,1,1] #jitter in pixels
info['minVal'] = 0
info['maxVal'] = info['halfnDots']
info['startVal'] = info['halfnDots']
info['nUp'] = 1
info['nDown'] = 3
info['feedback'] = True

#Create a window
if DEBUG == True:
    myWin = visual.Window(size = (800,800), monitor = 'testMonitor', units = 'pix', fullscr = False, allowGUI = False)
    #info['displayT'] = 30 #500ms at 60Hz
    info['ISI'] = 30
if DEBUG == False:
    myWin = visual.Window(size = [1024,768], screen = 1, useFBO = True, fullscr = True, allowGUI = False, monitor = 'Topaz', units = 'pix')
    #info['displayT'] = 34 #400ms at 85Hz
    #info['displayT'] = 68 #800ms at 85Hz
    info['ISI'] = 34
    bits = crs.BitsSharp(myWin, mode = 'bits++')

#fixation
fixation = visual.Circle(myWin, radius = 3.5, lineColor = 'black', fillColor = 'black', contrast = 0.3)

#sounds
beep = sound.Sound('800', secs=0.01)
boop = sound.Sound('300', secs=0.1)

#Checking Responses
def checkCorrect (keys):
    for key in keys:
        if key in ['q', 'escape']:
            core.quit()
        elif key in ['num_1', 'num_3']:
            if (key in ['num_1']) and order==1:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1 #subject perceives the first image as more symmetrical
            if (key in ['num_3']) and order==1:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0 #subject perceives the second image as more symmetrical
            if (key in ['num_1']) and order==2:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0
            if (key in ['num_3']) and order==2:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1
            else:
                print "hit 1 or 3 (or q) (you hit %s)" %key
                return None

#Create multiple staircases in order to interleave them
stairs = []
conditions = ['shiftHigh', 'shiftLow'] #one starting with no jitter and one starting with max jitter

for thisCond in conditions:
    thisInfo = copy.copy(info)
    if thisCond == 'shiftHigh':
        thisInfo['cond'] = 'shiftHigh'
        thisInfo['nShift'] = info['maxVal']
        thisInfo['startVal'] = info['maxVal']
    if thisCond == 'shiftLow':
        thisInfo['cond'] = 'shiftLow'
        thisInfo['startVal'] = info['minVal']
        thisInfo['nShift'] = info['minVal']
    
    #staircase info
    #info for this staircase
    thisStair = dataTest.StairHandler(startVal = thisInfo['startVal'],
                                                nReversals = info['nReversals'],
                                                stepSizes = info['stepSizes'],
                                                stepType = 'lin',
                                                nTrials = info['nTrials'],
                                                nUp = info['nUp'],
                                                nDown = info['nDown'],
                                                extraInfo = thisInfo,
                                                minVal = info['minVal'],
                                                maxVal = info['maxVal'],
                                                        )
    stairs.append(thisStair)

startMessage = visual.TextStim(myWin, pos = (0.0, -150), height = 25, rgb = 1,
                text = 'Please press 1 or 3 to indicate whether the first or second image appears more symmetrical. Press any key to continue')
startMessage.draw()
fixation.draw()
myWin.flip()
event.waitKeys()
myWin.flip()
#create dot patterns
#test
leftxy, rightxy, xy = symFunctions10.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], info['jitter'])
#foil
foilLeftxy, foilRightxy, foilxy = symFunctions10.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], info['maxVal'])

test = visual.ElementArrayStim(myWin, fieldPos = (info['size']/2.0*-1, info['size']/2.0*-1), elementTex = None, elementMask = 'circle', nElements = info['halfnDots']*2,
                        sizes = info['dotSize'], xys = xy, colors = 'white')
foil = visual.ElementArrayStim(myWin, fieldPos = (info['size']/2.0*-1, info['size']/2.0*-1), elementTex = None, elementMask = 'circle', nElements = info['halfnDots']*2, 
                        sizes = info['dotSize'] , xys = foilxy, colors = 'white')

#Go through the trials
for trialN in range(info['nTrials']):
    random.shuffle(stairs)
    for thisStair in stairs:
        thisShift = info['maxVal'] - thisStair.next()
        print 'shift', thisShift
        
        #create new patterns
        leftxy, rightxy, xy = symFunctions10.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], jitter = 0)
        foilLeftxy, foilRightxy, foilxy = symFunctions10.createIrregular(info['size'], info['halfnDots'], info['dotSize'], info['bubbleSize'], jitter = 0)
        xy, noiseIndex = symFunctions10.createNoise(thisShift, xy, info['bubbleSize'], info['dotSize'], info['size'])
        foilxy, foilNoiseIndex = symFunctions10.createNoise(info['maxVal'], foilxy, info['bubbleSize'], info['dotSize'], info['size'])
        
        signalIndex = [x for x in range(info['halfnDots']*2) if x not in noiseIndex]
        #foilSignalIndex = [x for x in range(halfnDots*2) if x not in foilNoiseIndex]
        
        test.setXYs(xy)
        foil.setXYs(foilxy)
        
        #give the dots different starting lives
        lives = []
        foilLives = []
        for n in range(info['halfnDots']*2):
            start = random.randint(0, info['lifeTime'])
            foilStart = random.randint(0, info['lifeTime'])
            lives.append(start)
            foilLives.append(foilStart)
        lives = np.array(lives)
        foilLives = np.array(foilLives)
        thisStair.addOtherData('Lives', lives)
        thisStair.addOtherData('FoilLives', foilLives)
        #randomise foil and target
        order = random.randint(1,2)
        #present stimuli
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
                #bits.beep(freq = 1200, dur = 0.3)
        
        for frameN in range(info['displayT']):
            lives += 1
            foilLives +=1
            #make dots limited lifetime
            for n in range(info['halfnDots']):
                if n in signalIndex:
                    if lives[n] > info['lifeTime']:
                        newXY = symFunctions10.relocate('left', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'], mirror = True)
                        xy[n] = newXY
                        xy[n+info['halfnDots']] = [info['size'] - newXY[0], newXY[1]]
                        lives[n] = 0
                if n in noiseIndex:
                    if lives[n] > info['lifeTime']:
                        newXY = symFunctions10.relocate('left', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                        xy[n] = newXY
                        lives[n] = 0
                    if lives[n+info['halfnDots']] > info['lifeTime']:
                        newXY = symFunctions10.relocate('right', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                        xy[n + info['halfnDots']] = newXY
                        lives[n+info['halfnDots']] = 0
                if foilLives[n] > info['lifeTime']:
                    newXY = symFunctions10.relocate('left', foilxy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                    foilxy[n] = newXY
                    foilLives[n] = 0
                if foilLives[n + info['halfnDots']] > info['lifeTime']:
                    newXY = symFunctions10.relocate('right', foilxy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                    foilxy[n + info['halfnDots']] = newXY
                    foilLives[n+info['halfnDots']] = 0
            test.setXYs(xy)
            foil.setXYs(foilxy)
            
            if order == 1:
                test.draw()
                fixation.draw()
                myWin.flip()
            if order == 2:
                foil.draw()
                fixation.draw()
                myWin.flip()
            myWin.getMovieFrame()
        myWin.saveMovieFrames('Stim1.jpg')

        #event.waitKeys()
        #ISI
        for frameN in range(info['ISI']):
            fixation.draw()
            myWin.flip()
        #second interval
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
            #bits.beep(freq = 1200, dur = 0.3)
        for frameN in range(info['displayT']):
            lives += 1
            foilLives +=1
            #make dots limited lifetime
            for n in range(info['halfnDots']):
                if n in signalIndex:
                    if lives[n] > info['lifeTime']:
                        newXY = symFunctions10.relocate('left', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'], mirror = True)
                        xy[n] = newXY
                        xy[n+info['halfnDots']] = [info['size'] - newXY[0], newXY[1]]
                        lives[n] = 0
                if n in noiseIndex:
                    if lives[n] > info['lifeTime']:
                        newXY = symFunctions10.relocate('left', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                        xy[n] = newXY
                        lives[n] = 0
                    if lives[n+info['halfnDots']] > info['lifeTime']:
                        newXY = symFunctions10.relocate('right', xy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                        xy[n + info['halfnDots']] = newXY
                        lives[n+info['halfnDots']] = 0
                if foilLives[n] > info['lifeTime']:
                    newXY = symFunctions10.relocate('left', foilxy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                    foilxy[n] = newXY
                    foilLives[n] = 0
                if foilLives[n + info['halfnDots']] > info['lifeTime']:
                    newXY = symFunctions10.relocate('right', foilxy, info['size'], info['minDist'], info['bubbleSize'], info['dotSize'])
                    foilxy[n + info['halfnDots']] = newXY
                    foilLives[n+info['halfnDots']] = 0
            test.setXYs(xy)
            foil.setXYs(foilxy)
            
            if order == 1:
                foil.draw()
                fixation.draw()
                myWin.flip()
            if order == 2:
                test.draw()
                fixation.draw()
                myWin.flip()
            myWin.getMovieFrame()
        myWin.saveMovieFrames('Stim2.jpg')
        #event.waitKeys()
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

        #clear the screen
        fixation.draw()
        myWin.flip()
        
        #collect response
        thisResp = None
        while thisResp==None:
            keys = event.waitKeys()
            thisResp = checkCorrect(keys)
        #print 'thisResp', thisResp
        
        thisStair.addData(thisResp)
                
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

#Saving data
if not os.path.isdir('irrStatic_%s' %info['participant']):
    os.mkdir('irrStatic_%s' %info['participant'])

for thisStair in stairs:
    fName = 'irrStatic_%s//irrStatic_LL%d_%dms_%ddots_%s_%s_%s' %(thisStair.extraInfo['participant'], thisStair.extraInfo['lifeTime'], thisStair.extraInfo['displayT'],
                    thisStair.extraInfo['density'],thisStair.extraInfo['cond'], thisStair.extraInfo['participant'], info['dateStr'])
    
    thisStair.saveAsExcel(fileName = fName, sheetName = 'RawData', matrixOnly = False, appendFile = True)
    thisStair.saveAsPickle(fName)
core.quit()







