#Function library for motion symmetry - Nov 2015

from psychopy import visual, event, core
import random, copy
import numpy as np

def createIrregular(size, halfnDots, dotSize, bubbleSize, jitter=0):
    """ Create and irregular symmetrical pattern:
        size: the size of the field in pixels
        nDots: number of dots of total dots divided by 2
        dotSize: size of dots in pixels
        bubbleSize: the buffer needed around each dot (dotSize will be added to this)
        """
    
    if jitter > (bubbleSize):#-(dotSize/2)):
        print 'ERROR: jitter is larger than maximum bubble size'
    if (((bubbleSize*2)**2)*(halfnDots*2)) > ((size**2)/2):
        print 'ERROR: You have run out of space'
    
    leftx = []
    lefty = []
    rightx = []
    righty = []

    minDist = (bubbleSize*2) + (dotSize/2)

    #First x,y
    x1 = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
    y1 = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
    x1 = float(x1)
    y1 = float(y1)
    leftx.append(x1)
    lefty.append(y1)

    dots = 1
    restarts = 0
    
    while dots < halfnDots:
        x2 = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
        y2 = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
        x2 = float(x2)
        y2 = float(y2)
        temp = []
        for m in range(len(leftx)):
            hypot = np.hypot(x2 - leftx[m], y2 - lefty[m])
            temp.append(hypot)
        counter = 0
        while any(i < minDist for i in temp):
            counter +=1
            if counter > 1000: #start the whole pattern again
                leftx = []
                lefty = []
                x1 = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
                y1 = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
                leftx.append(x1)
                lefty.append(y1)
                dots = 1
                counter = 0
                restarts +=1
            if restarts > 1000:
                print 'ERROR: no fitting pattern found'
            x2 = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
            y2 = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
            temp = []
            for m in range(len(leftx)):
                hypot = np.hypot(x2 - leftx[m], y2 - lefty[m])
                temp.append(hypot)
        leftx.append(x2)
        lefty.append(y2)
        dots += 1
       
    leftxy = np.array([leftx,lefty]).transpose()

    #Create mirror
    for n in range(len(leftxy)):
        rightx.append(size - leftxy[n][0])
        righty.append(leftxy[n][1])
    rightxy = np.array([rightx,righty]).transpose()
    
    xy = np.concatenate((leftxy, rightxy))
    
    #Add jitter to all dots
    if jitter !=0:
        for n in range (halfnDots):
            leftTheta = np.random.randint(0, np.pi*2)
            rightTheta = np.random.randint(0, np.pi*2)
            leftxy[n][0] += np.cos(leftTheta)*jitter
            leftxy[n][1] += np.sin(leftTheta)*jitter
            rightxy[n][0] += np.cos(rightTheta)*jitter
            rightxy[n][1] += np.sin(rightTheta)*jitter
    
    xy = np.concatenate((leftxy, rightxy))

    return rightxy, leftxy, xy

def createNoise(nNoise, xy, bubbleSize, dotSize, size, ins = 0, outs = 0):
    """Create positional shift noise. nNoise is the number of dots that will be unmatched.
    This values is halved as moving one dot, creates two unmatched dots.
    nNoise: The number of unmatched dots
    xy: The original xy coords
    """
    
    #nNoise /= 2 #nNoise needs to be halved as moving one dot creates two noise dots
    minDist = (bubbleSize*2) + (dotSize/2)
    if ins != 0:
        inIndex = random.sample(ins, nNoise/2)
        outIndex = random.sample(outs, nNoise - (nNoise/2))
        evenCounter = 1
        for n in range(nNoise):
            if evenCounter%2 == 0:
                for n in range(len(inIndex)):
                    if inIndex[n] < len(xy)/2.0:
                        newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                    if inIndex[n] >= len(xy)/2.0:
                        newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                    xy[inIndex[n]] = newXY
            
            if evenCounter%2 != 0:
                for n in range(len(outIndex)):
                    if outIndex[n] < len(xy)/2.0:
                        newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                    if outIndex[n] >= len(xy)/2.0:
                        newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                    xy[outIndex[n]] = newXY
            evenCounter += 1
        return xy, inIndex, outIndex
    if ins == 0:
        index = random.sample(range(len(xy)/2), nNoise)
        for n in range(len(index)):
            #random x coord in the correct half
            if index[n] < len(xy)/2.0:
                newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
            if index[n] >= len(xy)/2.0:
                newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
            xy[index[n]] = newXY
        return xy, index

def linMotion(nPatterns, condition, inc, size, halfnDots, dotSize, bubbleSize, jitter, nNoise, displayT):
    """Pre-creates linear motion patterns
    nPatterns: The number of different patterns required
    condition: 'in', 'out' or 'inOut'
    """
    
    #pre-create the patterns
    patterns = {}
    ins=0
    outs=0
    minDist = (bubbleSize*2) + (dotSize/2)
    for patternN in range (nPatterns):
        leftxy, rightxy, xy = createIrregular(size, halfnDots, dotSize, bubbleSize, jitter)
        
        if condition == 'inOut':
            outs = random.sample(range(halfnDots), halfnDots/2)
            ins = [x for x in range(halfnDots) if x not in outs]
            xy2, inNoiseIndex, outNoiseIndex = createNoise(nNoise, xy, bubbleSize, dotSize, size, ins=ins, outs = outs)
            outSignalIndex = [x for x in range(halfnDots/2) if x not in outNoiseIndex]
            inSignalIndex = [x for x in range(halfnDots/2) if x not in inNoiseIndex]
            
        if condition != 'inOut':
            xy2, noiseIndex = createNoise(nNoise, xy, bubbleSize, dotSize, size, ins=ins, outs = outs)
            signalIndex = [x for x in range(halfnDots*2) if x not in noiseIndex]
        
        originalXY = copy.copy(xy)

        #shift the dots a bit in their bubbles
        for n in range(halfnDots):
            move = np.random.randint(bubbleSize*-1+dotSize, bubbleSize-dotSize)
            xy2[n][0] += move
            xy2[n+halfnDots][0] -= move
        
        patterns[patternN] = {}
        
        #create complete frame sequence
        for frameN in range(displayT):#(bubbleSize*2-dotSize):
            if condition == 'out':
                #make the left half move
                for n in range(halfnDots):
                    #move left half
                    xy2[n][0] -= inc
                    #make right half move
                    xy2[n+halfnDots][0] +=inc
                
                    #move symmetrical dots
                    if n in signalIndex:
                        #loop if it goes too far
                        if xy2[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize, mirror = True)
                            originalXY[n] = newXY
                            xy2[n] = newXY
                            xy2[n][0] = xy2[n][0] + bubbleSize - dotSize/2.0

                            originalXY[n+halfnDots] = [size - xy2[n][0], xy2[n][1]]
                            xy2[n+halfnDots] = [size - xy2[n][0], xy2[n][1]]

                #move noise dots
                for n in range(halfnDots):
                    if n in noiseIndex:
                        #left
                        if xy2[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[n] = newXY
                            xy2[n] = newXY
                            xy2[n][0] = xy2[n][0] + bubbleSize - dotSize/2.0

                        #right
                        if xy2[n+halfnDots][0] > (originalXY[n+halfnDots][0] + bubbleSize - dotSize/2.0):
                            #random x coord in the correct half
                            newXY = relocate('right', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[n+halfnDots] = newXY
                            xy2[n+halfnDots] = newXY
                            xy2[n+halfnDots][0] = originalXY[n+halfnDots][0] - bubbleSize + dotSize/2.0

                patterns[patternN][frameN] = copy.copy(xy2)
                
            if condition == 'in':
                for n in range(halfnDots):
                    #move left half
                    xy2[n][0] += inc
                    #make right half move
                    xy2[n+halfnDots][0] -=inc
                    #move symmetrical dots
                    if n in signalIndex:
                        #loop if it goes too far
                        if xy2[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize, mirror = True)
                            originalXY[n] = newXY
                            xy2[n] = newXY
                            xy2[n][0] = xy2[n][0] - bubbleSize + dotSize/2.0

                            originalXY[n+halfnDots] = [size - xy2[n][0], xy2[n][1]]
                            xy2[n+halfnDots] = [size - xy2[n][0], xy2[n][1]]

                #move noise dots
                for n in range(halfnDots):
                    if n in noiseIndex:
                        #left
                        if xy2[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[n] = newXY
                            xy2[n] = newXY
                            xy2[n][0] = xy2[n][0] - bubbleSize + dotSize/2.0

                        #right
                        if xy2[n+halfnDots][0] < (originalXY[n+halfnDots][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('right', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[n+halfnDots] = newXY
                            xy2[n+halfnDots] = newXY
                            xy2[n+halfnDots][0] = originalXY[n+halfnDots][0] + bubbleSize - dotSize/2.0
                patterns[patternN][frameN] = copy.copy(xy2)

            if condition == 'inOut':
                #outward dots
                for n in range(len(outs)):
                    if n in outSignalIndex:
                        xy2[outs[n]][0] -= inc
                        #make right half move
                        xy2[outs[n]+halfnDots][0] +=inc
                        #loop if it goes too far
                        if xy2[outs[n]][0] < (originalXY[outs[n]][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize, mirror = True)
                            originalXY[outs[n]] = newXY
                            xy2[outs[n]] = newXY
                            xy2[outs[n]][0] = xy2[outs[n]][0] + bubbleSize - dotSize/2.0

                            originalXY[outs[n]+halfnDots] = [size - xy2[outs[n]][0], xy2[outs[n]][1]]
                            xy2[outs[n]+halfnDots] = [size - xy2[outs[n]][0], xy2[outs[n]][1]]

                #move outwards noise dots
                for n in range(len(outs)):
                    #print outs
                    if n in outNoiseIndex:
                        xy2[outs[n]][0] -= inc
                        #make right half move
                        xy2[outs[n]+halfnDots][0] +=inc
                        #left
                        if xy2[outs[n]][0] < (originalXY[outs[n]][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[outs[n]] = newXY
                            xy2[outs[n]] = newXY
                            xy2[outs[n]][0] = xy2[outs[n]][0] + bubbleSize - dotSize/2.0

                        #right
                        if xy2[outs[n]+halfnDots][0] > (originalXY[outs[n]+halfnDots][0] + bubbleSize - dotSize/2.0):
                            newXY = relocate('right', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[outs[n]+halfnDots] = newXY
                            xy2[outs[n]+halfnDots] = newXY
                            xy2[outs[n]+halfnDots][0] = originalXY[outs[n]+halfnDots][0] - bubbleSize + dotSize/2.0

                #move inwards dots
                for n in range(len(ins)):
                                    #move symmetrical dots
                    if n in inSignalIndex:
                        xy2[ins[n]][0] += inc
                        xy2[ins[n]+halfnDots][0] -=inc
                        #loop if it goes too far
                        if xy2[ins[n]][0] > (originalXY[ins[n]][0] + bubbleSize - dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize, mirror = True)
                            originalXY[ins[n]] = newXY
                            xy2[ins[n]] = newXY
                            xy2[ins[n]][0] = xy2[ins[n]][0] - bubbleSize + dotSize/2.0

                            originalXY[ins[n]+halfnDots] = [size - xy2[ins[n]][0], xy2[ins[n]][1]]
                            xy2[ins[n]+halfnDots] = [size - xy2[ins[n]][0], xy2[ins[n]][1]]

                #move inward noise dots
                for n in range(len(ins)):
                    if n in inNoiseIndex:
                        xy2[ins[n]][0] += inc
                        xy2[ins[n]+halfnDots][0] -=inc
                        #left
                        if xy2[ins[n]][0] > (originalXY[ins[n]][0] + bubbleSize - dotSize/2.0):
                            newXY = relocate('left', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[ins[n]] = newXY
                            xy2[ins[n]] = newXY
                            xy2[ins[n]][0] = xy2[ins[n]][0] - bubbleSize + dotSize/2.0

                        #right
                        if xy2[ins[n]+halfnDots][0] < (originalXY[ins[n]+halfnDots][0] - bubbleSize + dotSize/2.0):
                            newXY = relocate('right', xy2, size, minDist, bubbleSize, dotSize)
                            originalXY[ins[n]+halfnDots] = newXY
                            xy2[ins[n]+halfnDots] = newXY
                            xy2[ins[n]+halfnDots][0] = originalXY[ins[n]+halfnDots][0] + bubbleSize - dotSize/2.0
                patterns[patternN][frameN] = copy.copy(xy2)
    return patterns

def randMotion(nPatterns, inc, size, halfnDots, dotSize, bubbleSize, jitter, nNoise, displayT):
    randPatterns = {}
    minDist = (bubbleSize*2) + (dotSize/2)#+1 #extra pixel to make sure there is always a one pixel gap
    
    for patternN in range(nPatterns):
        #make a starting pattern
        rightxy, leftxy, xy = createIrregular(size, halfnDots, dotSize, bubbleSize, jitter)
        #add Noise
        xy, noiseIndex = createNoise(nNoise, xy, bubbleSize, dotSize, size)
        signalIndex = [x for x in range(halfnDots*2) if x not in noiseIndex]
        originalxy = copy.copy(xy)

        theta = np.empty(halfnDots)
        cos = np.empty(halfnDots)
        sin = np.empty(halfnDots)

        for dotN in range(halfnDots):
            theta[dotN] = np.random.rand() * np.pi * 2
            cos[dotN] = np.cos(theta[dotN]) * inc 
            sin[dotN] = np.sin(theta[dotN]) * inc

            #shift the dots a bit in their bubbles
            move = float(np.random.randint(bubbleSize*-1 + dotSize, bubbleSize - dotSize))
            xy[:halfnDots,0][dotN] += np.sin(theta[dotN]) * move
            xy[:halfnDots,1][dotN] += np.cos(theta[dotN]) * move
            xy[halfnDots:,0][dotN] += np.sin(theta[dotN]) * move *-1
            xy[halfnDots:,1][dotN] += np.cos(theta[dotN]) * move
        
        randPatterns[patternN] = {}
        
        for frameN in range(displayT):#(bubbleSize*2-dotSize):
            #move dots
            for n in range(halfnDots):
                xy[n][0] += sin[n]
                xy[n][1] += cos[n]
                xy[n+halfnDots][0] += sin[n] *-1
                xy[n+halfnDots][1] += cos[n]
                
                if n in signalIndex:
                    #move if it goes too far
                    hypot = np.hypot(xy[n][0] - originalxy[n][0], xy[n][1] - originalxy[n][1])
                    hypot2 = np.hypot(xy[n+halfnDots][0] - originalxy[n+halfnDots][0], xy[n+halfnDots][1] - xy[n+halfnDots][1])
                    if hypot >= (bubbleSize - dotSize/2.0) or hypot2 >= (bubbleSize - dotSize/2.0):
                        newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize, mirror = True)
                        originalxy[n] = newXY
                        originalxy[n+halfnDots] = [size - newXY[0], newXY[1]]
                        xy[n] = newXY
                        
                        xy[n][0] -= sin[n]*(bubbleSize - dotSize/2.0)
                        xy[n][1] -= cos[n]*(bubbleSize - dotSize/2.0)
                        xy[n+halfnDots] = [size - newXY[0], newXY[1]]
                        xy[n+halfnDots][0] -= (sin[n]*-1)*(bubbleSize - dotSize/2.0)
                        xy[n+halfnDots][1] -= cos[n]*(bubbleSize - dotSize/2.0)
                        
            #move noise dots
            for n in range(halfnDots):
                if n in noiseIndex:
                    #left
                    hypot = np.hypot(xy[n][0] - originalxy[n][0], xy[n][1] - originalxy[n][1])
                    if hypot >= (bubbleSize - dotSize/2.0):
                        newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                        originalxy[n] = newXY
                        xy[n] = newXY
                        xy[n][0] -= sin[n]*(bubbleSize- dotSize/2.0)
                        xy[n][1] -= cos[n]*(bubbleSize- dotSize/2.0)

                    #right
                    hypot2 = np.hypot(xy[n+halfnDots][0] - originalxy[n+halfnDots][0], xy[n+halfnDots][1] - originalxy[n+halfnDots][1])
                    if hypot2 >= (bubbleSize - dotSize/2.0):
                        newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                        originalxy[n+halfnDots] = newXY
                        xy[n+halfnDots] = newXY
                        xy[n+halfnDots][0] -= (sin[n]*-1)*(bubbleSize- dotSize/2.0)
                        xy[n+halfnDots][1] -= cos[n]*(bubbleSize- dotSize/2.0)

            randPatterns[patternN][frameN] = copy.copy(xy)
    return randPatterns

def relocate(half, xy, size, minDist, bubbleSize, dotSize, mirror = False):
        #random x coord in the correct half
    tempx = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
    if half == 'right':
        tempx = size - np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
    #random y coord
    tempy = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
    hypots = []
    for m in range(len(xy)):
        hypot = np.hypot(tempx - xy[m][0], tempy - xy[m][1])
        hypots.append(hypot)
        if mirror == True:
            hypot2 = np.hypot((size - tempx) - xy[m][0], tempy - xy[m][1])
            hypots.append(hypot2)
    counter = 0
    while any(i < minDist for i in hypots): #when it's too close to other dots keep moving it
        #print hypots
        counter += 1
        if counter == 50000:
            print 'ERROR: No space'
            core.quit()
        tempx = np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
        if half == 'right':
            tempx = size - np.random.randint(0+bubbleSize/2+dotSize*2,size/2-(bubbleSize/2)-dotSize*2)
        tempy = np.random.randint(0+bubbleSize/2+dotSize*2,size-(bubbleSize/2)-dotSize*2)
        hypots = []
        for m in range(len(xy)):
            hypot = np.hypot(tempx - xy[m][0], tempy - xy[m][1])
            hypots.append(hypot)
            if mirror == True:
                hypot2 = np.hypot((size - tempx) - xy[m][0], tempy - xy[m][1])
                hypots.append(hypot2)
    newXY = [tempx, tempy]
    return newXY

def motionOnly(nPatterns, condition, size, halfnDots, dotSize, bubbleSize, nNoise, inc, displayT, jitter = 0):
    minDist = (bubbleSize*2) + (dotSize/2)
    patterns = {}
    for patternN in range(nPatterns):
        patterns[patternN] = {}
        leftxy, rightxy, xy = createIrregular(size, halfnDots, dotSize, bubbleSize, jitter)
        #100% positional noise
        xy, Index = createNoise(halfnDots, xy, bubbleSize, dotSize, size)
        originalXY = copy.copy(xy)
        #divide into noise and signal
        noiseIndex1 = random.sample(range(halfnDots), nNoise/2)
        noiseIndex2 = random.sample(range(halfnDots, (halfnDots*2)+1), nNoise - nNoise/2)
        noiseIndex = noiseIndex1 + noiseIndex2
        signalIndex = [x for x in range(halfnDots*2) if x not in noiseIndex]

        #for in out divide into two directions
        outs1 = random.sample(range(halfnDots), halfnDots/2)
        outs2 = random.sample(range(halfnDots, (halfnDots*2)+1), halfnDots - halfnDots/2)
        outs = outs1 + outs2
        ins = [x for x in range(halfnDots*2) if x not in outs]

        #create random directions for noise
        theta = np.empty(halfnDots*2)
        cos = np.empty(halfnDots*2)
        sin = np.empty(halfnDots*2)

        for dotN in range(halfnDots*2):
            theta[dotN] = np.random.rand() * np.pi * 2
            #make sure noise doesn't move horizontally
            while theta[dotN] == np.pi/2.0 or theta[dotN] == (np.pi + np.pi/2.0):
                theta[dotN] = np.random.rand() * np.pi * 2
            cos[dotN] = np.cos(theta[dotN]) * inc 
            sin[dotN] = np.sin(theta[dotN]) * inc

        #shift the dots a bit in their bubbles
        for n in range(halfnDots*2):
            move = np.random.randint(bubbleSize*-1+dotSize, bubbleSize-dotSize)
            xy[n][0] += move

        for frameN in range(displayT):
            for n in range(halfnDots*2):
                #move symmetrical motion dots
                if n in signalIndex:
                    if n < halfnDots:
                        #move left half
                        if condition == 'out' or condition == 'left':
                            xy[n][0] -= inc
                            if xy[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                                newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                                originalXY[n] = newXY
                                xy[n][0] = newXY[0] + bubbleSize - dotSize/2.0
                                xy[n][1] = newXY[1]
                        if condition == 'in' or condition == 'right':
                            xy[n][0] += inc
                            if xy[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                                newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                                originalXY[n] = newXY
                                xy[n][0] = newXY[0] - bubbleSize + dotSize/2.0
                                xy[n][1] = newXY[1]
                        if condition == 'inOut':
                            if n in outs:
                                xy[n][0] -= inc
                                if xy[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                                    newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                                    originalXY[n] = newXY
                                    xy[n][0] = newXY[0] + bubbleSize - dotSize/2.0
                                    xy[n][1] = newXY[1]
                            if n in ins:
                                xy[n][0] += inc
                                if xy[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                                    newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                                    originalXY[n] = newXY
                                    xy[n][0] = newXY[0] - bubbleSize + dotSize/2.0
                                    xy[n][1] = newXY[1]

                    if n >= halfnDots:
                        #make right half move
                        if condition == 'out' or condition == 'right':
                            xy[n][0] += inc
                            if xy[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                                newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                                originalXY[n] = newXY
                                xy[n][0] = originalXY[n][0] - bubbleSize + dotSize/2.0
                                xy[n][1] = originalXY[n][1]
                        if condition == 'in' or condition == 'left':
                            xy[n][0] -= inc
                            if xy[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                                newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                                originalXY[n] = newXY
                                xy[n][0] = originalXY[n][0] + bubbleSize - dotSize/2.0
                                xy[n][1] = originalXY[n][1]
                        if condition == 'inOut':
                            if n in outs:
                                xy[n][0] += inc
                                if xy[n][0] > (originalXY[n][0] + bubbleSize - dotSize/2.0):
                                    newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                                    originalXY[n] = newXY
                                    xy[n][0] = originalXY[n][0] - bubbleSize + dotSize/2.0
                                    xy[n][1] = originalXY[n][1]
                            if n in ins:
                                xy[n][0] -= inc
                                if xy[n][0] < (originalXY[n][0] - bubbleSize + dotSize/2.0):
                                    newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                                    originalXY[n] = newXY
                                    xy[n][0] = originalXY[n][0] + bubbleSize - dotSize/2.0
                                    xy[n][1] = originalXY[n][1]

                if n in noiseIndex:
                    if n < halfnDots:
                        #move dots
                        xy[n][0] = xy[n][0] + sin[n]
                        xy[n][1] = xy[n][1] + cos[n]
                        #move if it goes too far
                        hypot = np.hypot(xy[n][0] - originalXY[n][0], xy[n][1] - originalXY[n][1])
                        if hypot >= (bubbleSize - dotSize/2.0): 
                            theta[n] = np.random.rand() * np.pi * 2
                            while theta[dotN] == np.pi/2.0 or theta[dotN] == (np.pi + np.pi/2.0):
                                theta[dotN] = np.random.rand() * np.pi * 2
                            cos[n] = np.cos(theta[n]) * inc 
                            sin[n] = np.sin(theta[n]) * inc
                            newXY = relocate('left', xy, size, minDist, bubbleSize, dotSize)
                            originalXY[n] = newXY
                            xy[n] = newXY
                            #move to edge of bubble
                            xy[n][0] -= sin[n]*(bubbleSize- dotSize/2.0)
                            xy[n][1] -= cos[n]*(bubbleSize- dotSize/2.0)
                            
                    if n >= halfnDots:
                        xy[n][0] = xy[n][0] + sin[n]
                        xy[n][1] = xy[n][1] + cos[n]
                        #move if it goes too far
                        hypot2 = np.hypot(xy[n][0] - originalXY[n][0], xy[n][1] - originalXY[n][1])
                        if hypot2 >= (bubbleSize - dotSize/2.0):
                            theta[n] = np.random.rand() * np.pi * 2
                            while theta[dotN] == np.pi/2.0 or theta[dotN] == (np.pi + np.pi/2.0):
                                theta[dotN] = np.random.rand() * np.pi * 2
                            cos[n] = np.cos(theta[n]) * inc 
                            sin[n] = np.sin(theta[n]) * inc
                            newXY = relocate('right', xy, size, minDist, bubbleSize, dotSize)
                            originalXY[n] = newXY
                            xy[n][0] = newXY[0]
                            xy[n][1] = newXY[1]
                            #move to edge of bubble
                            xy[n][0] -= sin[n]*(bubbleSize- dotSize/2.0)
                            xy[n][1] -= cos[n]*(bubbleSize- dotSize/2.0)
            patterns[patternN][frameN] = copy.copy(xy)
    return patterns
