#Symmetry detection - alternating between two patterns - March 2017

from psychopy import prefs
prefs.general['audioLib'] = ['pygame']
from psychopy import visual, event, core, data, gui, misc, sound, logging
from psychopy.hardware import crs
import symFunctions13, time, copy, random, os, cPickle
import numpy as np
import dataTest

DEBUG = True

#Create a dialog box for ppt information
try:
    info=misc.fromFile('symStaticLocSwap.pickle')
except:
    info = {'participant' : 'RJS', 'lifeTime' : 27, 'displayT':34, 'density':40}
info['dateStr']=time.strftime("%b%d_%H%M", time.localtime())
dlg = gui.DlgFromDict(info, title='Experiment', fixed=['dateStr'])
if dlg.OK:
    misc.toFile('symStaticLocSwap.pickle', info)
else:
    core.quit()

#Basic info
info['size'] = 320#350 #one side of the square
info['dotSize'] = 6 #diameter in pixels
#check number of dots is even
if info['density'] % 2 != 0:
    print 'ERROR: Must have an even number of dots'
    core.quit()
info['halfnDots'] = info['density']/2 #20 #half the total number of nots, to prevent odd number problems
info['bubbleSize'] = 10#12#14 #the size of the bubble each dot can move around in
info['jitter'] = 0
info['minDist'] = (info['bubbleSize']*2) + (info['dotSize']/2)

#Staircase information
info['nTrials'] = 75
info['nReversals'] = 8
info['stepSizes'] = [16,16,8,8,4,4,2,2] #jitter in pixels
info['minVal'] = 0
info['maxVal'] = info['halfnDots']*2
info['startVal'] = info['halfnDots']
info['nUp'] = 1
info['nDown'] = 3
info['feedback'] = True

#Create a window
if DEBUG == True:
    myWin = visual.Window(size = (800,800), monitor = 'testMonitor', units = 'pix', fullscr = False, allowGUI = False)
    #info['displayT'] = 30 #500ms at 60Hz
    info['ISI'] = 30
if DEBUG == False:
    myWin = visual.Window(size = [1024,768], screen = 1, useFBO = True, fullscr = True, allowGUI = False, monitor = 'Topaz', units = 'pix')
    #info['displayT'] = 34 #400ms at 85Hz
    #info['displayT'] = 68 #800ms at 85Hz
    info['ISI'] = 34
    #bits = crs.BitsSharp(myWin, mode = 'bits++')

#fixation
fixation = visual.Circle(myWin, radius = 3.5, lineColor = 'black', fillColor = 'black', contrast = 0.3)

#sounds
beep = sound.Sound('800', secs=0.01)
boop = sound.Sound('300', secs=0.1)

#Checking Responses
def checkCorrect (keys):
    for key in keys:
        if key in ['q', 'escape']:
            core.quit()
        elif key in ['num_1', 'num_3']:
            if (key in ['num_1']) and order==1:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1 #subject perceives the first image as more symmetrical
            if (key in ['num_3']) and order==1:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0 #subject perceives the second image as more symmetrical
            if (key in ['num_1']) and order==2:
#                incorrect1.draw()
                #if DEBUG == False and info['feedback'] == True:
                    #bits.beep(freq = 200, dur = 0.3)
                boop.play()
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 0
            if (key in ['num_3']) and order==2:
#                correct1.draw()
#                if DEBUG == False and info['feedback'] == True:
#                    bits.beep(freq = 1200, dur = 0.3)
                fixation.draw()
                myWin.flip()
                core.wait(0.5)
                return 1
            else:
                print "hit 1 or 3 (or q) (you hit %s)" %key
                return None

#Create multiple staircases in order to interleave them
stairs = []
conditions = ['shiftHigh', 'shiftLow'] #one starting with no jitter and one starting with max jitter

for thisCond in conditions:
    thisInfo = copy.copy(info)
    if thisCond == 'shiftHigh':
        thisInfo['cond'] = 'shiftHigh'
        thisInfo['nShift'] = info['maxVal']
        thisInfo['startVal'] = info['maxVal']
    if thisCond == 'shiftLow':
        thisInfo['cond'] = 'shiftLow'
        thisInfo['startVal'] = info['minVal']
        thisInfo['nShift'] = info['minVal']
    
    #staircase info
    #info for this staircase
    thisStair = dataTest.StairHandler(startVal = thisInfo['startVal'],
                                                nReversals = info['nReversals'],
                                                stepSizes = info['stepSizes'],
                                                stepType = 'lin',
                                                nTrials = info['nTrials'],
                                                nUp = info['nUp'],
                                                nDown = info['nDown'],
                                                extraInfo = thisInfo,
                                                minVal = info['minVal'],
                                                maxVal = info['maxVal'],
                                                        )
    stairs.append(thisStair)

startMessage = visual.TextStim(myWin, pos = (0.0, -150), height = 25, rgb = 1,
                text = 'Please press 1 or 3 to indicate whether the first or second image appears more symmetrical. Press any key to continue')
startMessage.draw()
fixation.draw()
myWin.flip()
event.waitKeys()
myWin.flip()

#create double density symmetrical patterns
rightxy, leftxy, xy = symFunctions13.createIrregular(info['size'], info['halfnDots']*2, info['dotSize'], info['bubbleSize'], info['jitter'])
foilRightxy, foilLeftxy, foilxy = symFunctions13.createIrregular(info['size'], info['halfnDots']*2, info['dotSize'], info['bubbleSize'], info['jitter'])

#create test pattern
xy1right = rightxy[:len(rightxy)/2]
xy1left = leftxy[:len(leftxy)/2]
xy1 = np.concatenate([xy1right, xy1left])
xy2right = rightxy[len(rightxy)/2:]
xy2left = leftxy[len(leftxy)/2:]
xy2 = np.concatenate([xy2right, xy2left])

#creat foil pattern
foilxy1right = foilRightxy[:len(foilRightxy)/2]
foilxy1left = foilLeftxy[:len(foilLeftxy)/2]
foilxy1 = np.concatenate([foilxy1right, foilxy1left])
foilxy2right = foilRightxy[len(foilRightxy)/2:]
foilxy2left = foilLeftxy[len(foilLeftxy)/2:]
foilxy2 = np.concatenate([foilxy2right, foilxy2left])

test = visual.ElementArrayStim(myWin, fieldPos = (info['size']/2.0*-1, info['size']/2.0*-1), elementTex = None, elementMask = 'circle', nElements = info['halfnDots']*2,
                        sizes = info['dotSize'], xys = xy1, colors = 'white')
foil = visual.ElementArrayStim(myWin, fieldPos = (info['size']/2.0*-1, info['size']/2.0*-1), elementTex = None, elementMask = 'circle', nElements = info['halfnDots']*2,
                        sizes = info['dotSize'], xys = foilxy1, colors = 'white')

#Go throught the trials
#Go through the trials
for trialN in range(info['nTrials']):
    random.shuffle(stairs)
    for thisStair in stairs:
        thisShift = info['maxVal'] - thisStair.next()
        print 'shift', thisShift
        
        #create Test
        #create double density symmetrical pattern
        rightxy, leftxy, xy = symFunctions13.createIrregular(info['size'], info['halfnDots']*2, info['dotSize'], info['bubbleSize'], info['jitter'])
        #add noise
        nNoise = thisShift
        if nNoise != 0 and (info['density']-nNoise) != 0:
            xy1nNoise = nNoise /2
            xy2nNoise = nNoise - (nNoise/2)
            
            xy, NoiseIndex = symFunctions13.createNoise(nNoise, xy, info['bubbleSize'], info['dotSize'], info['size'])

            leftNoiseIndex = NoiseIndex
            rightNoiseIndex = np.array(NoiseIndex)+info['halfnDots']*2

            leftSignalIndex = [x for x in range(info['halfnDots']*2) if x not in leftNoiseIndex]
            rightSignalIndex = [x for x in np.array(range(info['halfnDots']*2))+info['halfnDots']*2 if x not in rightNoiseIndex]

            xy1leftNoiseIndex = leftNoiseIndex[:xy1nNoise]
            xy1rightNoiseIndex = rightNoiseIndex[:xy1nNoise]

            xy1leftSignalIndex = leftSignalIndex[:info['halfnDots']-xy1nNoise]
            xy1rightSignalIndex = rightSignalIndex[:info['halfnDots']-xy1nNoise]
            xy1signalIndex = np.concatenate([xy1leftSignalIndex, xy1rightSignalIndex])

            xy2leftNoiseIndex = leftNoiseIndex[xy2nNoise:]
            xy2rightNoiseIndex = rightNoiseIndex[xy2nNoise:]

            xy2leftSignalIndex = leftSignalIndex[info['halfnDots']-xy2nNoise:]
            xy2rightSignalIndex = rightSignalIndex[info['halfnDots']-xy2nNoise:]
            xy2signalIndex = np.concatenate([xy2leftSignalIndex, xy2rightSignalIndex])

            #mix and match the noise to make it more noisy
            xy1noiseIndex = np.concatenate([xy1leftNoiseIndex, xy2rightNoiseIndex])
            xy2noiseIndex = np.concatenate([xy2leftNoiseIndex, xy1rightNoiseIndex])
            xy1Index = np.concatenate([xy1noiseIndex, xy1signalIndex])
            xy2Index = np.concatenate([xy2noiseIndex, xy2signalIndex])

            xy1 = xy[xy1Index]
            xy2 = xy[xy2Index]
            
        if info['density']-nNoise == 0:
            xy1nNoise = nNoise /2
            xy2nNoise = nNoise - (nNoise/2)
            
            xy, NoiseIndex = symFunctions13.createNoise(nNoise, xy, info['bubbleSize'], info['dotSize'], info['size'])

            leftNoiseIndex = NoiseIndex
            rightNoiseIndex = np.array(NoiseIndex)+info['halfnDots']*2

            xy1leftNoiseIndex = leftNoiseIndex[:xy1nNoise]
            xy1rightNoiseIndex = rightNoiseIndex[:xy1nNoise]

            xy2leftNoiseIndex = leftNoiseIndex[xy2nNoise:]
            xy2rightNoiseIndex = rightNoiseIndex[xy2nNoise:]

            #mix and match the noise to make it more noisy
            xy1Index = np.concatenate([xy1leftNoiseIndex, xy2rightNoiseIndex])
            xy2Index = np.concatenate([xy2leftNoiseIndex, xy1rightNoiseIndex])

            xy1 = xy[xy1Index]
            xy2 = xy[xy2Index]

        if nNoise == 0:
            xy1right = rightxy[:len(rightxy)/2]
            xy1left = leftxy[:len(leftxy)/2]
            xy1 = np.concatenate([xy1right, xy1left])
            xy2right = rightxy[len(rightxy)/2:]
            xy2left = leftxy[len(leftxy)/2:]
            xy2 = np.concatenate([xy2right, xy2left])

        #create Foil
        #create double density symmetrical pattern
        foilrightxy, foilleftxy, foilxy = symFunctions13.createIrregular(info['size'], info['halfnDots']*2, info['dotSize'], info['bubbleSize'], info['jitter'])
        #add noise
        nNoise = info['maxVal']
        if nNoise != 0 and (info['density']-nNoise) != 0:
            foilxy1nNoise = nNoise /2
            foilxy2nNoise = nNoise - (nNoise/2)
            
            foilxy, foilNoiseIndex = symFunctions13.createNoise(nNoise, foilxy, info['bubbleSize'], info['dotSize'], info['size'])

            foilleftNoiseIndex = foilNoiseIndex
            foilrightNoiseIndex = np.array(foilNoiseIndex)+info['halfnDots']*2

            foilleftSignalIndex = [x for x in range(info['halfnDots']*2) if x not in foilleftNoiseIndex]
            foilrightSignalIndex = [x for x in np.array(range(info['halfnDots']*2))+info['halfnDots']*2 if x not in foilrightNoiseIndex]

            foilxy1leftNoiseIndex = foilleftNoiseIndex[:foilxy1nNoise]
            foilxy1rightNoiseIndex = foilrightNoiseIndex[:foilxy1nNoise]

            foilxy1leftSignalIndex = foilleftSignalIndex[:info['halfnDots']-foilxy1nNoise]
            foilxy1rightSignalIndex = foilrightSignalIndex[:info['halfnDots']-foilxy1nNoise]
            foilxy1signalIndex = np.concatenate([foilxy1leftSignalIndex, foilxy1rightSignalIndex])

            foilxy2leftNoiseIndex = foilleftNoiseIndex[foilxy2nNoise:]
            foilxy2rightNoiseIndex = foilrightNoiseIndex[foilxy2nNoise:]

            foilxy2leftSignalIndex = foilleftSignalIndex[info['halfnDots']-foilxy2nNoise:]
            foilxy2rightSignalIndex = foilrightSignalIndex[info['halfnDots']-foilxy2nNoise:]
            foilxy2signalIndex = np.concatenate([foilxy2leftSignalIndex, foilxy2rightSignalIndex])

            #mix and match the noise to make it more noisy
            foilxy1noiseIndex = np.concatenate([foilxy1leftNoiseIndex, foilxy2rightNoiseIndex])
            foilxy2noiseIndex = np.concatenate([foilxy2leftNoiseIndex, foilxy1rightNoiseIndex])
            foilxy1Index = np.concatenate([foilxy1noiseIndex, foilxy1signalIndex])
            foilxy2Index = np.concatenate([foilxy2noiseIndex, foilxy2signalIndex])

            foilxy1 = foilxy[xy1Index]
            foilxy2 = foilxy[xy2Index]
            
        if info['density']-nNoise == 0:
            foilxy1nNoise = nNoise /2
            foilxy2nNoise = nNoise - (nNoise/2)
            
            foilxy, foilNoiseIndex = symFunctions13.createNoise(nNoise, foilxy, info['bubbleSize'], info['dotSize'], info['size'])

            foilleftNoiseIndex = foilNoiseIndex
            foilrightNoiseIndex = np.array(foilNoiseIndex)+info['halfnDots']*2

            foilxy1leftNoiseIndex = foilleftNoiseIndex[:foilxy1nNoise]
            foilxy1rightNoiseIndex = foilrightNoiseIndex[:foilxy1nNoise]

            foilxy2leftNoiseIndex = foilleftNoiseIndex[foilxy2nNoise:]
            foilxy2rightNoiseIndex = foilrightNoiseIndex[foilxy2nNoise:]

            #mix and match the noise to make it more noisy
            foilxy1Index = np.concatenate([foilxy1leftNoiseIndex, foilxy2rightNoiseIndex])
            foilxy2Index = np.concatenate([foilxy2leftNoiseIndex, foilxy1rightNoiseIndex])

            foilxy1 = foilxy[foilxy1Index]
            foilxy2 = foilxy[foilxy2Index]

        if nNoise == 0:
            foilxy1right = foilrightxy[:len(foilrightxy)/2]
            foilxy1left = foilleftxy[:len(foilleftxy)/2]
            foilxy1 = np.concatenate([foilxy1right, foilxy1left])
            foilxy2right = foilrightxy[len(foilrightxy)/2:]
            foilxy2left = foilleftxy[len(foilleftxy)/2:]
            foilxy2 = np.concatenate([foilxy2right, foilxy2left])

        xy = copy.copy(xy1)
        foilxy = copy.copy(foilxy1)
        test.setXYs(xy)
        foil.setXYs(foilxy)
        
        #to start off show pattern 1
        list = np.ones(info['halfnDots']) #are we drawing from xy1 or xy2
        foilList = np.ones(info['halfnDots'])

        #give the dots different starting lives
        lives = []
        foilLives = []
        for n in range(info['halfnDots']*2):
            start = random.randint(0, info['lifeTime'])
            foilStart = random.randint(0, info['lifeTime'])
            lives.append(start)
            foilLives.append(foilStart)
        lives = np.array(lives)
        foilLives = np.array(foilLives)
        thisStair.addOtherData('Lives', lives)
        thisStair.addOtherData('FoilLives', foilLives)
#        lives = np.ones(info['halfnDots']*2)
#        foilLives = np.ones(info['halfnDots']*2)


        #randomise foil and target
        order = random.randint(1,2)
        #present stimuli
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
                #bits.beep(freq = 1200, dur = 0.3)

        for frameN in range(info['displayT']):
            lives += 1
            foilLives += 1
            #make dots limited lifetime
            for n in range(info['halfnDots']):
                if lives[n] > info['lifeTime']:
                    if list[n] == 1: # if it's xy1 switch to xy2
                        xy[n] = xy2[n]
                        xy[n+info['halfnDots']] = xy2[n+info['halfnDots']]
                        list[n] = 2
                        lives[n] = 0
                        lives[n+info['halfnDots']] = 0
                    else:
                        xy[n] = xy1[n]
                        xy[n+info['halfnDots']] = xy1[n+info['halfnDots']]
                        list[n] = 1
                        lives[n] = 0
                        lives[n+info['halfnDots']] = 0
                if foilLives[n] > info['lifeTime']:
                    if list[n] == 1: # if it's xy1 switch to xy2
                        foilxy[n] = foilxy2[n]
                        foilxy[n+info['halfnDots']] = foilxy2[n+info['halfnDots']]
                        foilList[n] = 2
                        foilLives[n] = 0
                        foilLives[n+info['halfnDots']] = 0
                    else:
                        foilxy[n] = foilxy1[n]
                        foilxy[n+info['halfnDots']] = foilxy1[n+info['halfnDots']]
                        foilList[n] = 1
                        foilLives[n] = 0
                        foilLives[n+info['halfnDots']] = 0
            test.setXYs(xy)
            foil.setXYs(foilxy)

            if order == 1:
                test.draw()
                fixation.draw()
                myWin.flip()
            if order == 2:
                foil.draw()
                fixation.draw()
                myWin.flip()
            myWin.getMovieFrame()
        myWin.saveMovieFrames('Stim1.jpg')

        #event.waitKeys()
        #ISI
        for frameN in range(info['ISI']):
            fixation.draw()
            myWin.flip()
        #second interval
        beep.play()
        #if DEBUG == False and info['feedback'] == True:
            #bits.beep(freq = 1200, dur = 0.3)

        for frameN in range(info['displayT']):
            lives += 1
            foilLives += 1
            #make dots limited lifetime
            for n in range(info['halfnDots']):
                if lives[n] > info['lifeTime']:
                    if list[n] == 1: # if it's xy1 switch to xy2
                        xy[n] = xy2[n]
                        xy[n+info['halfnDots']] = xy2[n+info['halfnDots']]
                        list[n] = 2
                        lives[n] = 0
                        lives[n+info['halfnDots']] = 0
                    else:
                        xy[n] = xy1[n]
                        xy[n+info['halfnDots']] = xy1[n+info['halfnDots']]
                        list[n] = 1
                        lives[n] = 0
                        lives[n+info['halfnDots']] = 0
                if foilLives[n] > info['lifeTime']:
                    if list[n] == 1: # if it's xy1 switch to xy2
                        foilxy[n] = foilxy2[n]
                        foilxy[n+info['halfnDots']] = foilxy2[n+info['halfnDots']]
                        foilList[n] = 2
                        foilLives[n] = 0
                        foilLives[n+info['halfnDots']] = 0
                    else:
                        foilxy[n] = foilxy1[n]
                        foilxy[n+info['halfnDots']] = foilxy1[n+info['halfnDots']]
                        foilList[n] = 1
                        foilLives[n] = 0
                        foilLives[n+info['halfnDots']] = 0
            test.setXYs(xy)
            foil.setXYs(foilxy)

            if order == 1:
                foil.draw()
                fixation.draw()
                myWin.flip()
            if order == 2:
                test.draw()
                fixation.draw()
                myWin.flip()
            myWin.getMovieFrame()
        myWin.saveMovieFrames('Stim2.jpg')
        #event.waitKeys()
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

        #clear the screen
        fixation.draw()
        myWin.flip()
        
        #collect response
        thisResp = None
        while thisResp==None:
            keys = event.waitKeys()
            thisResp = checkCorrect(keys)
        #print 'thisResp', thisResp
        
        thisStair.addData(thisResp)
                
        for keys in event.getKeys():
            if keys in ['q', 'escape']:
                myWin.close()
                core.quit()

#Saving data
if not os.path.isdir('irrStatic_%s' %info['participant']):
    os.mkdir('irrStatic_%s' %info['participant'])

for thisStair in stairs:
    fName = 'irrStatic_%s//irrStaticLocSwap_LL%d_%dms_%ddots_%s_%s_%s' %(thisStair.extraInfo['participant'], thisStair.extraInfo['lifeTime'], thisStair.extraInfo['displayT'],
                    thisStair.extraInfo['density'],thisStair.extraInfo['cond'], thisStair.extraInfo['participant'], info['dateStr'])
    
    thisStair.saveAsExcel(fileName = fName, sheetName = 'RawData', matrixOnly = False, appendFile = True)
    thisStair.saveAsPickle(fName)
core.quit()










